.dspic
.include "p33fj06gs101a.inc"
.include "lcd_philips.inc"

SPI_SDO equ 3
SPI_CLK equ 1
SPI_CS equ 4
LCD_RESET equ 0

.macro lcd_command(a)
  mov #a, w10
  call send_command
.endm

.macro lcd_data(a)
  mov #a, w10
  call send_data
.endm

.org 0
  goto start

.org 256
start:
  mov #0x800, SP

  ;; Setup I/O Ports
  mov #((1<<LCD_RESET)|(1<<SPI_CLK)|(1<<SPI_CS)|(1<<SPI_SDO))^0xffff, w0
  mov w0, TRISB
  mov #(1<<LCD_RESET)|(1<<SPI_CLK)|(1<<SPI_CS), w0
  mov w0, LATB

  mov #0x3^0xffff, w0
  mov w0, TRISA
  mov #0x3, w0
  mov w0, LATA

  ;; Documentation says this has to be done.  I'm not seeing that I need it?
  ;mov #OSCCON, w3
  ;mov #0x46, w4
  ;mov #0x57, w5
  ;mov.b w4, [w3]
  ;mov.b w5, [w3]

  ;mov.b w4, [w3]
  ;mov.b w5, [w3]

  ;; Setup SPI
  ; SDI is on RP2
  mov #SDI1R2, w0
  mov w0, RPINR20

  ; SDO is on RP3
  mov #(0x7<<8), w0
  mov w0, RPOR1      ; controls RP2, RP3

  ; SCLK is on RP1
  mov #(0x8<<8), w0
  mov w0, RPOR0      ; controls RP0, RP1

  mov #(1<<MSTEN)|(1<<CKP)|(0x6<<2)|(0x2), w0
  mov w0, SPI1CON1
  mov #(1<<SPIEN), w0
  mov w0, SPI1STAT

  ;; Reset LCD
  ;bclr LATB, #LCD_RESET
  ;call delay
  ;bset LATB, #LCD_RESET
  ;call delay
  ;call delay

  mov #0x01, w5
  mov w5, w0
  xor #0xff, w0
  and LATB
  call delay
  mov #0x01, w5
  mov w5, w0
  ior LATB
  call delay
  call delay

init_philips:
  lcd_command(SLEEPOUT)
  lcd_command(SETCON)
  lcd_data(0x40)
  lcd_command(COLMOD)
  lcd_data(0x03)
  lcd_command(MADCTL)
  lcd_data(0xc8)
  lcd_command(DISPON)

  call draw_image

  bclr LATA, #0

main:
  bra main

draw_init:
  lcd_command(PASET)
  ;lcd_data(0)
  ;lcd_data(131)
  lcd_data(50)
  lcd_data(97)

  lcd_command(CASET)
  ;lcd_data(0)
  ;lcd_data(131)
  lcd_data(50)
  lcd_data(97)
  return

draw_image:
  call draw_init

  ;mov.w #(132*132/2)*3, w7
  mov #(48*48/2), w7
  lcd_command(RAMWR)
repeat_draw:
  lcd_data(0x00)
  lcd_data(0xf0)
  lcd_data(0x0f)
  dec w7, w7
  bra nz, repeat_draw
  return

;; send_command(w10)
send_command:
  ; manually clock out bit 8 (7 to 0 is hardware)
  ; bit 8 is a 0 for command
  bclr LATB, #SPI_SDO
  bclr LATB, #SPI_CS             ; /CS=selected
  call clock_bit

  mov w10, SPI1BUF
send_command_wait_spi:
  btss SPI1STAT, #SPIRBF
  bra send_command_wait_spi

  bset LATB, #SPI_CS             ; /CS=selected
  return

;; send_data(w10)
send_data:
  ; manually clock out bit 8 (7 to 0 is hardware)
  ; bit 8 is a 1 for data
  bset LATB, #SPI_SDO
  bclr LATB, #SPI_CS             ; /CS=selected
  call clock_bit
  bclr LATB, #SPI_SDO            ; probably not needed, but doesn't hurt

  mov w10, SPI1BUF
send_data_wait_spi:
  btss SPI1STAT, #SPIRBF
  bra send_data_wait_spi

  bset LATB, #SPI_CS             ; /CS=selected
  return

clock_bit:
  bclr SPI1STAT, #SPIEN          ; DISABLE part of SPI so we can use PORTB
  bclr LATB, #SPI_CLK            ; SPI_CLK=1
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  bset LATB, #SPI_CLK            ; SPI_CLK=1
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  nop
  bset SPI1STAT, #SPIEN          ; use hardware SPI again
  return

delay:
  mov #0xffff, w10
delay_loop:
  dec w10, w10
  bra nz, delay_loop
  return

.org __FICD
  dc32 0xffcf



